@@ -3,36 +3,33 @@
 import vobject
 import urllib
 from pywebdav.lib.errors import DAV_NotFound, DAV_Forbidden
-from trytond.model import ModelView, ModelSQL
 from trytond.tools import reduce_ids
 from trytond.transaction import Transaction
 from trytond.cache import Cache
-from trytond.pool import Pool
+from trytond.pool import Pool, PoolMeta
+
+__all__ = ['Collection']
+__metaclass__ = PoolMeta
 
 
-class Collection(ModelSQL, ModelView):
+class Collection:
+    __name__ = "webdav.collection"
+    _todo_cache = Cache('webdav_collection.todo')
 
-    _name = "webdav.collection"
-
-    @Cache('webdav_collection.todo')
-    def todo(self, uri, calendar_id=False):
+    @classmethod
+    def todo(cls, uri, calendar_id=False):
         '''
         Return the todo id in the uri
-
-        :param uri: the uri
-        :param calendar_id: the calendar id
-        :return: todo id
-            or None if there is no todo
         '''
-        todo_obj = Pool().get('calendar.todo')
+        Todo = Pool().get('calendar.todo')
 
         if uri and uri.startswith('Calendars/'):
             calendar, todo_uri = (uri[10:].split('/', 1) + [None])[0:2]
             if not calendar_id:
-                calendar_id = self.calendar(uri)
+                calendar_id = cls.calendar(uri)
                 if not calendar_id:
                     return None
-            todo_ids = todo_obj.search([
+            todo_ids = Todo.search([
                 ('calendar', '=', calendar_id),
                 ('uuid', '=', todo_uri[:-4]),
                 ('parent', '=', None),
@@ -40,12 +37,10 @@ class Collection(ModelSQL, ModelView):
             if todo_ids:
                 return todo_ids[0]
 
-    def _caldav_filter_domain_todo(self, filter):
+    @classmethod
+    def _caldav_filter_domain_todo(cls, filter):
         '''
         Return a domain for caldav filter on todo
-
-        :param filter: the DOM Element of filter
-        :return: a list for domain
         '''
         res = []
         if not filter:
@@ -98,78 +93,81 @@ class Collection(ModelSQL, ModelView):
                         continue
                     if uri:
                         uri = urllib.unquote_plus(uri)
-                    todo_id = self.todo(uri)
+                    todo_id = cls.todo(uri)
                     if todo_id:
                         ids.append(todo_id)
             return [('id', 'in', ids)]
         return res
 
-    def get_childs(self, uri, filter=None, cache=None):
-        todo_obj = Pool().get('calendar.todo')
+    @classmethod
+    def get_childs(cls, uri, filter=None, cache=None):
+        Todo = Pool().get('calendar.todo')
 
-        res = super(Collection, self).get_childs(uri, filter=filter,
+        res = super(Collection, cls).get_childs(uri, filter=filter,
                 cache=cache)
 
         if uri and (uri not in ('Calendars', 'Calendars/')) and \
                 uri.startswith('Calendars/'):
-            calendar_id = self.calendar(uri)
+            calendar_id = cls.calendar(uri)
             if  calendar_id and not (uri[10:].split('/', 1) + [None])[1]:
-                domain = self._caldav_filter_domain_todo(filter)
-                todo_ids = todo_obj.search([
+                domain = cls._caldav_filter_domain_todo(filter)
+                todos = Todo.search([
                     ('calendar', '=', calendar_id),
                     domain,
                     ])
-                todos = todo_obj.browse(todo_ids)
                 if cache is not None:
                     cache.setdefault('_calendar', {})
-                    cache['_calendar'].setdefault(todo_obj._name, {})
-                    for todo_id in todo_ids:
-                        cache['_calendar'][todo_obj._name][todo_id] = {}
+                    cache['_calendar'].setdefault(Todo.__name__, {})
+                    for todo in todos:
+                        cache['_calendar'][Todo.__name__][todo.id] = {}
                 return res + [x.uuid + '.ics' for x in todos]
 
         return res
 
-    def get_resourcetype(self, uri, cache=None):
+    @classmethod
+    def get_resourcetype(cls, uri, cache=None):
         from pywebdav.lib.constants import COLLECTION, OBJECT
         if uri in ('Calendars', 'Calendars/'):
             return COLLECTION
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if calendar_id:
             if not (uri[10:].split('/', 1) + [None])[1]:
                 return COLLECTION
-            if self.todo(uri, calendar_id=calendar_id):
+            if cls.todo(uri, calendar_id=calendar_id):
                 return OBJECT
-        elif self.calendar(uri, ics=True):
+        elif cls.calendar(uri, ics=True):
             return OBJECT
-        return super(Collection, self).get_resourcetype(uri, cache=cache)
+        return super(Collection, cls).get_resourcetype(uri, cache=cache)
 
-    def get_contenttype(self, uri, cache=None):
-        if self.todo(uri) \
-                or self.calendar(uri, ics=True):
+    @classmethod
+    def get_contenttype(cls, uri, cache=None):
+        if (cls.todo(uri)
+                or cls.calendar(uri, ics=True)):
             return 'text/calendar'
-        return super(Collection, self).get_contenttype(uri, cache=cache)
+        return super(Collection, cls).get_contenttype(uri, cache=cache)
 
-    def get_creationdate(self, uri, cache=None):
-        todo_obj = Pool().get('calendar.todo')
+    @classmethod
+    def get_creationdate(cls, uri, cache=None):
+        Todo = Pool().get('calendar.todo')
 
         cursor = Transaction().cursor
 
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if not calendar_id:
-            calendar_id = self.calendar(uri, ics=True)
+            calendar_id = cls.calendar(uri, ics=True)
         if calendar_id and (uri[10:].split('/', 1) + [None])[1]:
 
-            todo_id = self.todo(uri, calendar_id=calendar_id)
+            todo_id = cls.todo(uri, calendar_id=calendar_id)
             if todo_id:
                 if cache is not None:
                     cache.setdefault('_calendar', {})
-                    cache['_calendar'].setdefault(todo_obj._name, {})
-                    ids = cache['_calendar'][todo_obj._name].keys()
+                    cache['_calendar'].setdefault(Todo.__name__, {})
+                    ids = cache['_calendar'][Todo.__name__].keys()
                     if todo_id not in ids:
                         ids.append(todo_id)
                     elif 'creationdate' in cache['_calendar'][
-                            todo_obj._name][todo_id]:
-                        return cache['_calendar'][todo_obj._name][
+                            Todo.__name__][todo_id]:
+                        return cache['_calendar'][Todo.__name__][
                             todo_id]['creationdate']
                 else:
                     ids = [todo_id]
@@ -179,39 +177,40 @@ class Collection(ModelSQL, ModelView):
                     red_sql, red_ids = reduce_ids('id', sub_ids)
                     cursor.execute('SELECT id, ' \
                             'EXTRACT(epoch FROM create_date) ' \
-                        'FROM "' + todo_obj._table + '" ' \
+                        'FROM "' + Todo.__table__ + '" ' \
                         'WHERE ' + red_sql, red_ids)
                     for todo_id2, date in cursor.fetchall():
                         if todo_id2 == todo_id:
                             res = date
                         if cache is not None:
-                            cache['_calendar'][todo_obj._name]\
+                            cache['_calendar'][Todo.__name__]\
                                     .setdefault(todo_id2, {})
-                            cache['_calendar'][todo_obj._name][
+                            cache['_calendar'][Todo.__name__][
                                 todo_id2]['creationdate'] = date
                 if res is not None:
                     return res
 
-        return super(Collection, self).get_creationdate(uri, cache=cache)
+        return super(Collection, cls).get_creationdate(uri, cache=cache)
 
-    def get_lastmodified(self, uri, cache=None):
-        todo_obj = Pool().get('calendar.todo')
+    @classmethod
+    def get_lastmodified(cls, uri, cache=None):
+        Todo = Pool().get('calendar.todo')
 
         cursor = Transaction().cursor
 
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if calendar_id and (uri[10:].split('/', 1) + [None])[1]:
-            todo_id = self.todo(uri, calendar_id=calendar_id)
+            todo_id = cls.todo(uri, calendar_id=calendar_id)
             if todo_id:
                 if cache is not None:
                     cache.setdefault('_calendar', {})
-                    cache['_calendar'].setdefault(todo_obj._name, {})
-                    ids = cache['_calendar'][todo_obj._name].keys()
+                    cache['_calendar'].setdefault(Todo.__name__, {})
+                    ids = cache['_calendar'][Todo.__name__].keys()
                     if todo_id not in ids:
                         ids.append(todo_id)
                     elif 'lastmodified' in cache['_calendar'][
-                            todo_obj._name][todo_id]:
-                        return cache['_calendar'][todo_obj._name][
+                            Todo.__name__][todo_id]:
+                        return cache['_calendar'][Todo.__name__][
                             todo_id]['lastmodified']
                 else:
                     ids = [todo_id]
@@ -224,7 +223,7 @@ class Collection(ModelSQL, ModelView):
                     cursor.execute('SELECT COALESCE(parent, id), ' \
                                 'MAX(EXTRACT(epoch FROM ' \
                                 'COALESCE(write_date, create_date))) ' \
-                            'FROM "' + todo_obj._table + '" ' \
+                            'FROM "' + Todo.__table__ + '" ' \
                             'WHERE ' + red_id_sql + ' ' \
                                 'OR ' + red_parent_sql + ' ' \
                             'GROUP BY parent, id', red_id_ids + red_parent_ids)
@@ -232,83 +231,85 @@ class Collection(ModelSQL, ModelView):
                         if todo_id2 == todo_id:
                             res = date
                         if cache is not None:
-                            cache['_calendar'][todo_obj._name]\
+                            cache['_calendar'][Todo.__name__]\
                                     .setdefault(todo_id2, {})
-                            cache['_calendar'][todo_obj._name][
+                            cache['_calendar'][Todo.__name__][
                                 todo_id2]['lastmodified'] = date
                 if res is not None:
                     return res
 
-        return super(Collection, self).get_lastmodified(uri, cache=cache)
+        return super(Collection, cls).get_lastmodified(uri, cache=cache)
 
-    def get_data(self, uri, cache=None):
-        todo_obj = Pool().get('calendar.todo')
+    @classmethod
+    def get_data(cls, uri, cache=None):
+        Todo = Pool().get('calendar.todo')
 
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if calendar_id:
             if not (uri[10:].split('/', 1) + [None])[1]:
                 raise DAV_NotFound
-            todo_id = self.todo(uri, calendar_id=calendar_id)
+            todo_id = cls.todo(uri, calendar_id=calendar_id)
             if not todo_id:
-                return super(Collection, self).get_data(uri, cache=cache)
-            ical = todo_obj.todo2ical(todo_id)
+                return super(Collection, cls).get_data(uri, cache=cache)
+            ical = Todo(todo_id).todo2ical()
             return ical.serialize()
 
-        return super(Collection, self).get_data(uri, cache=cache)
+        return super(Collection, cls).get_data(uri, cache=cache)
 
-    def put(self, uri, data, content_type, cache=None):
-        todo_obj = Pool().get('calendar.todo')
-        calendar_obj = Pool().get('calendar.calendar')
+    @classmethod
+    def put(cls, uri, data, content_type, cache=None):
+        pool = Pool()
+        Todo = pool.get('calendar.todo')
+        Calendar = pool.get('calendar.calendar')
 
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if calendar_id:
             if not (uri[10:].split('/', 1) + [None])[1]:
                 raise DAV_Forbidden
-            todo_id = self.todo(uri, calendar_id=calendar_id)
+            todo_id = cls.todo(uri, calendar_id=calendar_id)
             ical = vobject.readOne(data)
             if not hasattr(ical, 'vtodo'):
-                return super(Collection, self).put(uri, data, content_type)
+                return super(Collection, cls).put(uri, data, content_type)
 
             if not todo_id:
 
-                values = todo_obj.ical2values(None, ical, calendar_id)
-                todo_id = todo_obj.create(values)
-                todo = todo_obj.browse(todo_id)
-                calendar = calendar_obj.browse(calendar_id)
+                values = Todo.ical2values(None, ical, calendar_id)
+                todo = Todo.create(values)
+                calendar = Calendar(calendar_id)
                 return Transaction().cursor.database_name + '/Calendars/' + \
                         calendar.name + '/' + todo.uuid + '.ics'
             else:
-                values = todo_obj.ical2values(todo_id, ical, calendar_id)
-                todo_obj.write(todo_id, values)
+                values = Todo.ical2values(todo_id, ical, calendar_id)
+                Todo.write([Todo(todo_id)], values)
                 return
 
-        return super(Collection, self).put(uri, data, content_type)
+        return super(Collection, cls).put(uri, data, content_type)
 
-    def rm(self, uri, cache=None):
-        todo_obj = Pool().get('calendar.todo')
+    @classmethod
+    def rm(cls, uri, cache=None):
+        Todo = Pool().get('calendar.todo')
 
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if calendar_id:
             if not (uri[10:].split('/', 1) + [None])[1]:
                 raise DAV_Forbidden
-            todo_id = self.todo(uri, calendar_id=calendar_id)
+            todo_id = cls.todo(uri, calendar_id=calendar_id)
             if todo_id:
                 try:
-                    todo_obj.delete(todo_id)
+                    Todo.delete([Todo(todo_id)])
                 except Exception:
                     raise DAV_Forbidden
                 return 200
-        return super(Collection, self).rm(uri, cache=cache)
+        return super(Collection, cls).rm(uri, cache=cache)
 
-    def exists(self, uri, cache=None):
+    @classmethod
+    def exists(cls, uri, cache=None):
         if uri in ('Calendars', 'Calendars/'):
             return 1
-        calendar_id = self.calendar(uri)
+        calendar_id = cls.calendar(uri)
         if calendar_id:
             if not (uri[10:].split('/', 1) + [None])[1]:
                 return 1
-            if self.todo(uri, calendar_id=calendar_id):
+            if cls.todo(uri, calendar_id=calendar_id):
                 return 1
-        return super(Collection, self).exists(uri, cache=cache)
-
-Collection()
+        return super(Collection, cls).exists(uri, cache=cache)
